@@ -1,39 +1,4 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Mozilla Firefox browser.
- *
- * The Initial Developer of the Original Code is
- * Benjamin Smedberg <benjamin@smedbergs.us>
- *
- * Portions created by the Initial Developer are Copyright (C) 2004
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+//@line 37 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
 
 const nsISupports            = Components.interfaces.nsISupports;
 
@@ -51,6 +16,7 @@ const nsIFactory             = Components.interfaces.nsIFactory;
 const nsIFileURL             = Components.interfaces.nsIFileURL;
 const nsIHttpProtocolHandler = Components.interfaces.nsIHttpProtocolHandler;
 const nsIInterfaceRequestor  = Components.interfaces.nsIInterfaceRequestor;
+const nsINetUtil             = Components.interfaces.nsINetUtil;
 const nsIPrefBranch          = Components.interfaces.nsIPrefBranch;
 const nsIPrefLocalizedString = Components.interfaces.nsIPrefLocalizedString;
 const nsISupportsString      = Components.interfaces.nsISupportsString;
@@ -61,11 +27,15 @@ const nsIWindowWatcher       = Components.interfaces.nsIWindowWatcher;
 const nsICategoryManager     = Components.interfaces.nsICategoryManager;
 const nsIWebNavigationInfo   = Components.interfaces.nsIWebNavigationInfo;
 const nsIBrowserSearchService = Components.interfaces.nsIBrowserSearchService;
+const nsICommandLineValidator = Components.interfaces.nsICommandLineValidator;
 
 const NS_BINDING_ABORTED = 0x804b0002;
 const NS_ERROR_WONT_HANDLE_CONTENT = 0x805d0001;
 const NS_ERROR_ABORT = Components.results.NS_ERROR_ABORT;
 
+const URI_INHERITS_SECURITY_CONTEXT = nsIHttpProtocolHandler
+                                        .URI_INHERITS_SECURITY_CONTEXT;
+
 function shouldLoadURI(aURI) {
   if (aURI && !aURI.schemeIs("chrome"))
     return true;
@@ -106,41 +76,132 @@ function resolveURIInternal(aCmdLine, aArgument) {
   return uri;
 }
 
+const OVERRIDE_NONE        = 0;
+const OVERRIDE_NEW_PROFILE = 1;
+const OVERRIDE_NEW_MSTONE  = 2;
+/**
+ * Determines whether a home page override is needed.
+ * Returns:
+ *  OVERRIDE_NEW_PROFILE if this is the first run with a new profile.
+ *  OVERRIDE_NEW_MSTONE if this is the first run with a build with a different
+ *                      Gecko milestone (i.e. right after an upgrade).
+ *  OVERRIDE_NONE otherwise.
+ */
 function needHomepageOverride(prefb) {
-  var savedmstone;
+  var savedmstone = null;
   try {
     savedmstone = prefb.getCharPref("browser.startup.homepage_override.mstone");
-  }
-  catch (e) {
-  }
+  } catch (e) {}
 
   if (savedmstone == "ignore")
-    return 0;
+    return OVERRIDE_NONE;
 
   var mstone = Components.classes["@mozilla.org/network/protocol;1?name=http"]
                          .getService(nsIHttpProtocolHandler).misc;
 
   if (mstone != savedmstone) {
+    // Bug 462254. Previous releases had a default pref to suppress the EULA
+    // agreement if the platform's installer had already shown one. Now with
+    // about:rights we've removed the EULA stuff and default pref, but we need
+    // a way to make existing profiles retain the default that we removed.
+    if (savedmstone)
+      prefb.setBoolPref("browser.rights.3.shown", true);
+    
     prefb.setCharPref("browser.startup.homepage_override.mstone", mstone);
-    // Return 1 if true if the pref didn't exist (i.e. new profile) or 2 for an upgrade
-    return (savedmstone ? 2 : 1);
+    return (savedmstone ? OVERRIDE_NEW_MSTONE : OVERRIDE_NEW_PROFILE);
   }
-  
-  // Return 0 if not a new profile and not an upgrade
-  return 0;
+
+  return OVERRIDE_NONE;
 }
 
-function openWindow(parent, url, target, features, args) {
+// Copies a pref override file into the user's profile pref-override folder,
+// and then tells the pref service to reload it's default prefs.
+function copyPrefOverride() {
+  try {
+    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
+                                .getService(Components.interfaces.nsIProperties);
+    const NS_APP_EXISTING_PREF_OVERRIDE = "ExistingPrefOverride";
+    var prefOverride = fileLocator.get(NS_APP_EXISTING_PREF_OVERRIDE,
+                                       Components.interfaces.nsIFile);
+    if (!prefOverride.exists())
+      return; // nothing to do
+
+    const NS_APP_PREFS_OVERRIDE_DIR     = "PrefDOverride";
+    var prefOverridesDir = fileLocator.get(NS_APP_PREFS_OVERRIDE_DIR,
+                                           Components.interfaces.nsIFile);
+
+    // Check for any existing pref overrides, and remove them if present
+    var existingPrefOverridesFile = prefOverridesDir.clone();
+    existingPrefOverridesFile.append(prefOverride.leafName);
+    if (existingPrefOverridesFile.exists())
+      existingPrefOverridesFile.remove(false);
+
+    prefOverride.copyTo(prefOverridesDir, null);
+
+    // Now that we've installed the new-profile pref override file,
+    // re-read the default prefs.
+    var prefSvcObs = Components.classes["@mozilla.org/preferences-service;1"]
+                               .getService(Components.interfaces.nsIObserver);
+    prefSvcObs.observe(null, "reload-default-prefs", null);
+  } catch (ex) {
+    Components.utils.reportError(ex);
+  }
+}
+
+// Flag used to indicate that the arguments to openWindow can be passed directly.
+const NO_EXTERNAL_URIS = 1;
+
+function openWindow(parent, url, target, features, args, noExternalArgs) {
   var wwatch = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                          .getService(nsIWindowWatcher);
 
-  var argstring;
-  if (args) {
-    argstring = Components.classes["@mozilla.org/supports-string;1"]
+  if (noExternalArgs == NO_EXTERNAL_URIS) {
+    // Just pass in the defaultArgs directly
+    var argstring;
+    if (args) {
+      argstring = Components.classes["@mozilla.org/supports-string;1"]
                             .createInstance(nsISupportsString);
-    argstring.data = args;
+      argstring.data = args;
+    }
+
+    return wwatch.openWindow(parent, url, target, features, argstring);
   }
-  return wwatch.openWindow(parent, url, target, features, argstring);
+  
+  // Pass an array to avoid the browser "|"-splitting behavior.
+  var argArray = Components.classes["@mozilla.org/supports-array;1"]
+                    .createInstance(Components.interfaces.nsISupportsArray);
+
+  // add args to the arguments array
+  var stringArgs = null;
+  if (args instanceof Array) // array
+    stringArgs = args;
+  else if (args) // string
+    stringArgs = [args];
+
+  if (stringArgs) {
+    // put the URIs into argArray
+    var uriArray = Components.classes["@mozilla.org/supports-array;1"]
+                       .createInstance(Components.interfaces.nsISupportsArray);
+    stringArgs.forEach(function (uri) {
+      var sstring = Components.classes["@mozilla.org/supports-string;1"]
+                              .createInstance(nsISupportsString);
+      sstring.data = uri;
+      uriArray.AppendElement(sstring);
+    });
+    argArray.AppendElement(uriArray);
+  } else {
+    argArray.AppendElement(null);
+  }
+
+  // Pass these as null to ensure that we always trigger the "single URL"
+  // behavior in browser.js's BrowserStartup (which handles the window
+  // arguments)
+  argArray.AppendElement(null); // charset
+  argArray.AppendElement(null); // referer
+  argArray.AppendElement(null); // postData
+  argArray.AppendElement(null); // allowThirdPartyFixup
+
+  return wwatch.openWindow(parent, url, target, features, argArray);
 }
 
 function openPreferences() {
@@ -161,27 +222,27 @@ function getMostRecentWindow(aType) {
   return wm.getMostRecentWindow(aType);
 }
 
-//@line 169 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 268 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
 
 // this returns the most recent non-popup browser window
 function getMostRecentBrowserWindow() {
   var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
 
-//@line 176 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 275 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
   var win = wm.getMostRecentWindow("navigator:browser", true);
 
   // if we're lucky, this isn't a popup, and we can just return this
-  if (win && !win.toolbar.visible) {
+  if (win && win.document.documentElement.getAttribute("chromehidden")) {
     var windowList = wm.getEnumerator("navigator:browser", true);
     // this is oldest to newest, so this gets a bit ugly
     while (windowList.hasMoreElements()) {
       var nextWin = windowList.getNext();
-      if (nextWin.toolbar.visible)
+      if (!nextWin.document.documentElement.getAttribute("chromehidden"))
         win = nextWin;
     }
   }
-//@line 201 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 300 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
 
   return win;
 }
@@ -241,8 +302,9 @@ var nsBrowserContentHandler = {
         !iid.equals(nsICommandLineHandler) &&
         !iid.equals(nsIBrowserHandler) &&
         !iid.equals(nsIContentHandler) &&
+        !iid.equals(nsICommandLineValidator) &&
         !iid.equals(nsIFactory))
-      throw Components.errors.NS_ERROR_NO_INTERFACE;
+      throw Components.results.NS_ERROR_NO_INTERFACE;
 
     return this;
   },
@@ -250,9 +312,10 @@ var nsBrowserContentHandler = {
   /* nsICommandLineHandler */
   handle : function bch_handle(cmdLine) {
     if (cmdLine.handleFlag("browser", false)) {
+      // Passing defaultArgs, so use NO_EXTERNAL_URIS
       openWindow(null, this.chromeURL, "_blank",
                  "chrome,dialog=no,all" + this.getFeatures(cmdLine),
-                 this.defaultArgs);
+                 this.defaultArgs, NO_EXTERNAL_URIS);
       cmdLine.preventDefault = true;
     }
 
@@ -266,14 +329,17 @@ var nsBrowserContentHandler = {
     if (remoteCommand != null) {
       try {
         var a = /^\s*(\w+)\(([^\)]*)\)\s*$/.exec(remoteCommand);
-        var remoteVerb = a[1].toLowerCase();
-        var remoteParams = [];
-        var sepIndex = a[2].lastIndexOf(",");
-        if (sepIndex == -1)
-          remoteParams[0] = a[2];
-        else {
-          remoteParams[0] = a[2].substring(0, sepIndex);
-          remoteParams[1] = a[2].substring(sepIndex + 1);
+        var remoteVerb;
+        if (a) {
+          remoteVerb = a[1].toLowerCase();
+          var remoteParams = [];
+          var sepIndex = a[2].lastIndexOf(",");
+          if (sepIndex == -1)
+            remoteParams[0] = a[2];
+          else {
+            remoteParams[0] = a[2].substring(0, sepIndex);
+            remoteParams[1] = a[2].substring(sepIndex + 1);
+          }
         }
 
         switch (remoteVerb) {
@@ -283,15 +349,26 @@ var nsBrowserContentHandler = {
           // openURL(<url>,new-window)
           // openURL(<url>,new-tab)
 
-          var uri = resolveURIInternal(cmdLine, remoteParams[0]);
+          // First param is the URL, second param (if present) is the "target"
+          // (tab, window)
+          var url = remoteParams[0];
+          var target = nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW;
+          if (remoteParams[1]) {
+            var targetParam = remoteParams[1].toLowerCase()
+                                             .replace(/^\s*|\s*$/g, "");
+            if (targetParam == "new-tab")
+              target = nsIBrowserDOMWindow.OPEN_NEWTAB;
+            else if (targetParam == "new-window")
+              target = nsIBrowserDOMWindow.OPEN_NEWWINDOW;
+            else {
+              // The "target" param isn't one of our supported values, so
+              // assume it's part of a URL that contains commas.
+              url += "," + remoteParams[1];
+            }
+          }
 
-          var location = nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW;
-          if (/new-window/.test(remoteParams[1]))
-            location = nsIBrowserDOMWindow.OPEN_NEWWINDOW;
-          else if (/new-tab/.test(remoteParams[1]))
-            location = nsIBrowserDOMWindow.OPEN_NEWTAB;
-
-          handURIToExistingBrowser(uri, location, cmdLine);
+          var uri = resolveURIInternal(cmdLine, url);
+          handURIToExistingBrowser(uri, target, cmdLine);
           break;
 
         case "xfedocommand":
@@ -299,20 +376,22 @@ var nsBrowserContentHandler = {
           if (remoteParams[0].toLowerCase() != "openbrowser")
             throw NS_ERROR_ABORT;
 
+          // Passing defaultArgs, so use NO_EXTERNAL_URIS
           openWindow(null, this.chromeURL, "_blank",
                      "chrome,dialog=no,all" + this.getFeatures(cmdLine),
-                     this.defaultArgs);
+                     this.defaultArgs, NO_EXTERNAL_URIS);
           break;
 
         default:
           // Somebody sent us a remote command we don't know how to process:
           // just abort.
-          throw NS_ERROR_ABORT;
+          throw "Unknown remote command.";
         }
 
         cmdLine.preventDefault = true;
       }
       catch (e) {
+        Components.utils.reportError(e);
         // If we had a -remote flag but failed to process it, throw
         // NS_ERROR_ABORT so that the xremote code knows to return a failure
         // back to the handling code.
@@ -353,12 +432,21 @@ var nsBrowserContentHandler = {
       // Handle the old preference dialog URL separately (bug 285416)
       if (chromeParam == "chrome://browser/content/pref/pref.xul") {
         openPreferences();
-      } else {
+        cmdLine.preventDefault = true;
+      } else try {
+        // only load URIs which do not inherit chrome privs
         var features = "chrome,dialog=no,all" + this.getFeatures(cmdLine);
-        openWindow(null, chromeParam, "_blank", features, "");
+        var uri = resolveURIInternal(cmdLine, chromeParam);
+        var netutil = Components.classes["@mozilla.org/network/util;1"]
+                                .getService(nsINetUtil);
+        if (!netutil.URIChainHasFlags(uri, URI_INHERITS_SECURITY_CONTEXT)) {
+          openWindow(null, uri.spec, "_blank", features);
+          cmdLine.preventDefault = true;
+        }
+      }
+      catch (e) {
+        Components.utils.reportError(e);
       }
-
-      cmdLine.preventDefault = true;
     }
     if (cmdLine.handleFlag("preferences", false)) {
       openPreferences();
@@ -373,7 +461,7 @@ var nsBrowserContentHandler = {
       cmdLine.preventDefault = true;
     }
 
-//@line 405 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 531 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
   },
 
   helpInfo : "  -browser            Open a browser window.\n",
@@ -386,39 +474,53 @@ var nsBrowserContentHandler = {
     var formatter = Components.classes["@mozilla.org/toolkit/URLFormatterService;1"]
                               .getService(Components.interfaces.nsIURLFormatter);
 
-    var pagesToLoad = "";
-    var overrideState = needHomepageOverride(prefb);
+    var overridePage = "";
+    var haveUpdateSession = false;
     try {
-      if (overrideState == 1) {
-        pagesToLoad = formatter.formatURLPref("startup.homepage_welcome_url");
-      }
-      else if (overrideState == 2) {
-        pagesToLoad = formatter.formatURLPref("startup.homepage_override_url");
-      }
-    }
-    catch (e) {
-    }
+      switch (needHomepageOverride(prefb)) {
+        case OVERRIDE_NEW_PROFILE:
+          // New profile
+          overridePage = formatter.formatURLPref("startup.homepage_welcome_url");
+          break;
+        case OVERRIDE_NEW_MSTONE:
+          // Existing profile, new build
+          copyPrefOverride();
 
-    var startpage = "";
+          // Check whether we have a session to restore. If we do, we assume
+          // that this is an "update" session.
+          var ss = Components.classes["@mozilla.org/browser/sessionstartup;1"]
+                             .getService(Components.interfaces.nsISessionStartup);
+          haveUpdateSession = ss.doRestore();
+          overridePage = formatter.formatURLPref("startup.homepage_override_url");
+          break;
+    }
+    } catch (ex) {}
+
+    // formatURLPref might return "about:blank" if getting the pref fails
+    if (overridePage == "about:blank")
+      overridePage = "";
+
+    var startPage = "";
     try {
       var choice = prefb.getIntPref("browser.startup.page");
-      if (choice == 1)
-        startpage = this.startPage;
+      if (choice == 1 || choice == 3)
+        startPage = this.startPage;
 
       if (choice == 2)
-        startpage = Components.classes["@mozilla.org/browser/global-history;2"]
+        startPage = Components.classes["@mozilla.org/browser/global-history;2"]
                               .getService(nsIBrowserHistory).lastPageVisited;
-    }
-    catch (e) {
+    } catch (e) {
+      Components.utils.reportError(e);
     }
 
-    if (startpage == "about:blank")
-      startpage = "";
+    if (startPage == "about:blank")
+      startPage = "";
 
-    if (pagesToLoad && startpage) pagesToLoad += "|";
-    pagesToLoad += startpage;
+    // Only show the startPage if we're not restoring an update session.
+    if (overridePage && startPage && !haveUpdateSession)
+      return overridePage + "|" + startPage;
 
-    return (pagesToLoad ?  pagesToLoad : "about:blank");
+    return overridePage || startPage || "about:blank";
   },
 
   get startPage() {
@@ -490,19 +592,27 @@ var nsBrowserContentHandler = {
       throw NS_ERROR_WONT_HANDLE_CONTENT;
     }
 
-    var parentWin;
-    try {
-      parentWin = context.getInterface(nsIDOMWindow);
-    }
-    catch (e) {
-    }
-
     request.QueryInterface(nsIChannel);
-    
-    openWindow(parentWin, request.URI, "_blank", null, null);
+    handURIToExistingBrowser(request.URI,
+      nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW, null);
     request.cancel(NS_BINDING_ABORTED);
   },
 
+  /* nsICommandLineValidator */
+  validate : function bch_validate(cmdLine) {
+    // Other handlers may use osint so only handle the osint flag if the url
+    // flag is also present and the command line is valid.
+    var osintFlagIdx = cmdLine.findFlag("osint", false);
+    var urlFlagIdx = cmdLine.findFlag("url", false);
+    if (urlFlagIdx > -1 && (osintFlagIdx > -1 ||
+        cmdLine.state == nsICommandLine.STATE_REMOTE_EXPLICIT)) {
+      var urlParam = cmdLine.getArgument(urlFlagIdx + 1);
+      if (cmdLine.length != urlFlagIdx + 2 || /firefoxurl:/.test(urlParam))
+        throw NS_ERROR_ABORT;
+      cmdLine.handleFlag("osint", false)
+    }
+  },
+
   /* nsIFactory */
   createInstance: function bch_CI(outer, iid) {
     if (outer != null)
@@ -551,28 +661,54 @@ var nsDefaultCommandLineHandler = {
     if (!iid.equals(nsISupports) &&
         !iid.equals(nsICommandLineHandler) &&
         !iid.equals(nsIFactory))
-      throw Components.errors.NS_ERROR_NO_INTERFACE;
+      throw Components.results.NS_ERROR_NO_INTERFACE;
 
     return this;
   },
 
+  // List of uri's that were passed via the command line without the app
+  // running and have already been handled. This is compared against uri's
+  // opened using DDE on Win32 so we only open one of the requests.
+  _handledURIs: [ ],
+//@line 742 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+
   /* nsICommandLineHandler */
   handle : function dch_handle(cmdLine) {
     var urilist = [];
 
+//@line 768 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+
     try {
       var ar;
       while ((ar = cmdLine.handleFlagWithParam("url", false))) {
-        urilist.push(resolveURIInternal(cmdLine, ar));
+        var found = false;
+        var uri = resolveURIInternal(cmdLine, ar);
+        // count will never be greater than zero except on Win32.
+        var count = this._handledURIs.length;
+        for (var i = 0; i < count; ++i) {
+          if (this._handledURIs[i].spec == uri.spec) {
+            this._handledURIs.splice(i, 1);
+            found = true;
+            cmdLine.preventDefault = true;
+            break;
+          }
+        }
+        if (!found) {
+          urilist.push(uri);
+          // The requestpending command line flag is only used on Win32.
+          if (cmdLine.handleFlag("requestpending", false) &&
+              cmdLine.state == nsICommandLine.STATE_INITIAL_LAUNCH)
+            this._handledURIs.push(uri)
+        }
       }
     }
     catch (e) {
       Components.utils.reportError(e);
     }
 
-    var count = cmdLine.length;
+    count = cmdLine.length;
 
-    for (var i = 0; i < count; ++i) {
+    for (i = 0; i < count; ++i) {
       var curarg = cmdLine.getArgument(i);
       if (curarg.match(/^-/)) {
         Components.utils.reportError("Warning: unrecognized command line flag " + curarg + "\n");
@@ -602,23 +738,19 @@ var nsDefaultCommandLineHandler = {
         }
       }
 
-      var speclist = [];
-      for (var uri in urilist) {
-        if (shouldLoadURI(urilist[uri]))
-          speclist.push(urilist[uri].spec);
-      }
-
-      if (speclist.length) {
+      var URLlist = urilist.filter(shouldLoadURI).map(function (u) u.spec);
+      if (URLlist.length) {
         openWindow(null, nsBrowserContentHandler.chromeURL, "_blank",
                    "chrome,dialog=no,all" + nsBrowserContentHandler.getFeatures(cmdLine),
-                   speclist.join("|"));
+                   URLlist);
       }
 
     }
     else if (!cmdLine.preventDefault) {
+      // Passing defaultArgs, so use NO_EXTERNAL_URIS
       openWindow(null, nsBrowserContentHandler.chromeURL, "_blank",
                  "chrome,dialog=no,all" + nsBrowserContentHandler.getFeatures(cmdLine),
-                 nsBrowserContentHandler.defaultArgs);
+                 nsBrowserContentHandler.defaultArgs, NO_EXTERNAL_URIS);
     }
   },
 
@@ -663,6 +795,15 @@ var Module = {
   },
     
   registerSelf: function mod_regself(compMgr, fileSpec, location, type) {
+    if (Components.classes["@mozilla.org/xre/app-info;1"]) {
+      // Don't register these if Firefox is launching a XULRunner application
+      const FIREFOX_UID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
+      var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
+                              .getService(Components.interfaces.nsIXULAppInfo);
+      if (appInfo.ID != FIREFOX_UID)
+        return;
+    }
+
     var compReg =
       compMgr.QueryInterface( Components.interfaces.nsIComponentRegistrar );
 
@@ -690,7 +831,9 @@ var Module = {
 
     registerType("text/html");
     registerType("application/vnd.mozilla.xul+xml");
-//@line 724 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 923 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
+    registerType("image/svg+xml");
+//@line 925 "/root/tmp/build-mozilla-firefox/mozilla/browser/components/nsBrowserContentHandler.js"
     registerType("text/rdf");
     registerType("text/xml");
     registerType("application/xhtml+xml");
@@ -715,6 +858,9 @@ var Module = {
     catMan.addCategoryEntry("command-line-handler",
                             "x-default",
                             dch_contractID, true, true);
+    catMan.addCategoryEntry("command-line-validator",
+                            "b-browser",
+                            bch_contractID, true, true);
   },
     
   unregisterSelf : function mod_unregself(compMgr, location, type) {
@@ -729,6 +875,8 @@ var Module = {
                                "m-browser", true);
     catMan.deleteCategoryEntry("command-line-handler",
                                "x-default", true);
+    catMan.deleteCategoryEntry("command-line-validator",
+                               "b-browser", true);
   },
 
   canUnload: function(compMgr) {
